<?php
/* Copyright (C) 2004		Rodolphe Quiedeville	<rodolphe@quiedeville.org>
 * Copyright (C) 2006-2007	Laurent Destailleur		<eldy@users.sourceforge.net>
 * Copyright (C) 2006-2012	Regis Houssin			<regis.houssin@inodbox.com>
 * Copyright (C) 2020       Sébastien Mas           <support@dolikom.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *       \file       htdocs/core/modules/societe/mod_codeclient_hamster.php
 *       \ingroup    societe
 *       \brief      Fichier de la classe des gestion lion des codes clients
 */

require_once DOL_DOCUMENT_ROOT.'/core/modules/societe/modules_societe.class.php';


/**
 *	Classe permettant la gestion monkey des codes tiers
 */
class mod_codeclient_Hamster extends ModeleThirdPartyCode
{
	/**
	 * @var string Nom du modele
	 * @deprecated
	 * @see name
	 */
	public $nom='Hamster';

	/**
	 * @var string model name
	 */
	public $name='Hamster';

	public $code_modifiable;				// Code modifiable

	public $code_modifiable_invalide;		// Code modifiable si il est invalide

	public $code_modifiable_null;			// Code modifiables si il est null

	public $code_null;						// Code facultatif

	/**
     * Dolibarr version of the loaded document
     * @var string
     */
	public $version = 'dolibarr';	    	// 'development', 'experimental', 'dolibarr'

	public $code_auto;                     // Numerotation automatique

	public $prefixIsRequired; // Le champ prefix du tiers doit etre renseigne quand on utilise {pre}


	/**
	 * 	Constructor
	 */
	public function __construct()
	{
		global $conf, $langs;
		$this->nom = "Hamster";
		$this->name = "Hamster";
		$this->version = "dolibarr";
		$this->code_null = 0;
		$this->code_modifiable = 1;
		$this->code_modifiable_invalide = 1;
		$this->code_modifiable_null = 1;
		$this->code_auto = 1;
		$this->prefixIsRequired = 0;

        if (! isset($conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER) || trim($conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER) == '') $conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER='5';
        $this->customeraccountancycodecharacternumber=$conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER;
        $this->supplieraccountancycodecharacternumber=$conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER;

        $langs->load('hamster');

	}


	/**
     *  Return description of module
	 *
	 *  @param	Translate	$langs	Object langs
	 *  @return string      		Description of module
	 */
	public function info($langs)
	{
		global $conf, $form;

        $tooltip='';
        $texte = '<form action="'.$_SERVER["PHP_SELF"].'" method="POST">';
        $texte.= '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
        $texte.= '<input type="hidden" name="action" value="setModuleOptions">';
        $texte.= '<input type="hidden" name="param3" value="COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER">';
        $texte.= '<table class="nobordernopadding" width="100%">';
        $s1= $form->textwithpicto('<input type="text" class="flat" size="2" name="value3" value="'.$conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER.'">', $tooltip, 1, 1);
        $texte.= '<tr><td>';
        $texte.= $langs->trans("ModuleCompanyCodeSupplier".$this->name, $s1)."<br>\n";
        $texte.= "<br>\n";
        // Remove special char if COMPANY_HAMSTER_REMOVE_SPECIAL is set to 1 or not set (default)
        if (! isset($conf->global->COMPANY_HAMSTER_REMOVE_SPECIAL) || ! empty($conf->global->$conf->global->COMPANY_HAMSTER_REMOVE_SPECIAL)) $texte.= $langs->trans('COMPANY_HAMSTER_REMOVE_SPECIAL').' = '.yn(1)."<br>\n";
        // Apply a regex replacement pattern on code if COMPANY_HAMSTER_CLEAN_REGEX is set. Value must be a regex with parenthesis. The part into parenthesis is kept, the rest removed.
        if (! empty($conf->global->COMPANY_HAMSTER_CLEAN_REGEX))  $texte.= $langs->trans('COMPANY_HAMSTER_CLEAN_REGEX').' = '.$conf->global->COMPANY_HAMSTER_CLEAN_REGEX."<br>\n";
        // Unique index on code if COMPANY_HAMSTER_UNIQUE_CODE is set to 1 or not set (default)
        if (! isset($conf->global->COMPANY_HAMSTER_UNIQUE_CODE) || ! empty($conf->global->COMPANY_HAMSTER_UNIQUE_CODE)) $texte.= $langs->trans('COMPANY_HAMSTER_UNIQUE_CODE').' = '.yn(1)."<br>\n";
        $texte.= '</td>';
        $texte.= '<td class="left">&nbsp; <input type="submit" class="button" value="'.$langs->trans("Modify").'" name="Button"></td>';
        $texte.= '</tr></table>';
        $texte.= '</form>';

        return $texte;
	}


	/**
	 * Return an example of result returned by getNextValue
	 *
	 * @param	Translate	$langs		Object langs
	 * @param	societe		$objsoc		Object thirdparty
	 * @param	int			$type		Type of third party (1:customer, 2:supplier, -1:autodetect)
	 * @return	string					Return string example
	 */
	public function getExample($langs, $objsoc = 0, $type = -1)
	{
		global $mysoc;

        $s= $langs->trans("ThirdPartyName").": ".$mysoc->name;
        $s.="<br>\n";

        if (! isset($conf->global->COMPANY_HAMSTER_REMOVE_SPECIAL)) $thirdpartylabelexample = preg_replace('/([^a-z0-9])/i', '', $mysoc->name);
        $s.="<br>\n";
        $s.=strtoupper(substr($thirdpartylabelexample,0, $this->customeraccountancycodecharacternumber))."00";
        return $s;
	}


/**
     * Return next value
     *
     * @param   Societe     $objsoc     Object third party
     * @param   int         $type       Client ou fournisseur (0:customer, 1:supplier)
     * @return  string                  Value if OK, '' if module not configured, <0 if KO
     */
    public function getNextValue($objsoc = 0, $type = -1)
    {
        global $db,$conf;

        dol_syslog("mod_codecompta_HAMSTER::get_NextValue for type=".$type);

        $field='';
        if ($type == 0)
        {
            $field = 'customer';
            //$where = ' AND client in (1,2)';
        }
        elseif ($type == 1)
        {
            $field = 'supplier';
            //$where = ' AND fournisseur = 1';
        }
        else return "type".$type;

        //dol_syslog("mod_codecompta_HAMSTER: for type=".$type." and field".$field);
        dol_syslog("mod_codecompta_HAMSTER:get_code for type=".$type." and field:".$field." & company=".(! empty($objsoc->name)?$objsoc->name:''));
        $numFinal=$this->get_code($db,$objsoc,$field);
        dol_syslog("mod_codecompta_HAMSTER:Code Client:".$numFinal);
        return  $numFinal;
    }


// phpcs:disable PEAR.NamingConventions.ValidFunctionName.ScopeNotCamelCaps
    /**
	 *  Set accountancy account code for a third party into this->code
	 *
	 *  @param	DoliDB	$db              Database handler
	 *  @param  Societe	$societe         Third party object
	 *  @param  int		$type			'customer' or 'supplier'
	 *  @return	int						>=0 if OK
     *				                     -1 CodeNotAvailable
     *		  							 -2 Disponibility
     * 		                             -3 Bad value for parameter type
	 */
	function get_code($db, $societe, $type='')
	{
        // phpcs:enable
        $i = 1;
        $codeclient='';
        dol_syslog("mod_codecompta_HAMSTER::get_code Début Fonction");

		if (is_object($societe))
		{
            dol_syslog("mod_codecompta_HAMSTER::get_code search code for type=".$type." & company=".(! empty($societe->name)?$societe->name:''));

			if ($type == 'customer') {
                $codetouse = $societe->name;
                $typeCcode = 0;
                $width = $this->supplieraccountancycodecharacternumber;
			}
			elseif ($type == 'supplier')
            {
                $codetouse = 'F'.$societe->name;
                $typeCcode = 1;
                $width = $this->customeraccountancycodecharacternumber;
            }
            else
            {
                $this->error = 'Bad value for parameter type';
                return -3;
            }

            // Remove special char if COMPANY_HAMSTER_REMOVE_SPECIAL is set to 1 or not set (default)
            if (! isset($conf->global->COMPANY_HAMSTER_REMOVE_SPECIAL) || ! empty($conf->global->COMPANY_HAMSTER_REMOVE_SPECIAL)) $codetouse=preg_replace('/([^a-z0-9])/i', '', $codetouse);
            // Apply a regex replacement pattern on code if COMPANY_DIGITARIA_CLEAN_REGEX is set. Value must be a regex with parenthesis. The part into parenthesis is kept, the rest removed.
            if (! empty($conf->global->COMPANY_HAMSTER_CLEAN_REGEX))	// Example: $conf->global->COMPANY_DIGITARIA_CLEAN_REGEX='^..(..)..';
            {
                $codetouse=preg_replace('/'.$conf->global->COMPANY_HAMSTER_CLEAN_REGEX.'/', '\1\2\3', $codetouse);
            }

            $codeclient = strtoupper(substr($codetouse, 0, $width-2)).'00';
            
            // Unique index on code if COMPANY_HAMSTER_UNIQUE_CODE is set to 1 or not set (default)
            if (! isset($conf->global->COMPANY_HAMSTER_UNIQUE_CODE) || ! empty($conf->global->COMPANY_HAMSTER_UNIQUE_CODE))
            {
				//echo $type.":".$codeclient.' ->'.$disponibility;
                $disponibility = $this->verif_dispo($db, $codeclient, $societe, $typeCcode);
                //echo  $codeclient.'-'.$typeCcode.'-'.$disponibility;
                while ($disponibility <> 0 && $i < 100) {
                    $widthsupplier = $this->supplieraccountancycodecharacternumber;
                    $widthcustomer = $this->customeraccountancycodecharacternumber;

                    if ($i <= 9) {
                        $a = 1;
                    }
                    if ($i >= 10 && $i <= 99) {
                        $a = 2;
                    }

                    if ($type == 'supplier') {
                        $codeclient = strtoupper(substr($codeclient, 0, $widthsupplier  - $a)) . $i;
                    //    echo "codes".$type.":".$codeclient;
                    } elseif ($type == 'customer') {
                        $codeclient = strtoupper(substr($codeclient, 0, $widthcustomer - $a)) . $i;
                    }
                    
                    $disponibility = $this->verif_dispo($db,  $codeclient, $societe, $typeCcode);
                    $i++;
                }
            } else {
                $disponibility == 0;
            }
		}

        if ($disponibility == 0)
        {
            dol_syslog("mod_codecompta_HAMSTER::get_code search code proposed=".$codeclient);
            return $codeclient; // return ok
        }
        else
        {
            return -2; // return ko
        }
	}


    /**
     *  Check validity of code according to its rules
     *
     *  @param  DoliDB      $db     Database handler
     *  @param  string      $code   Code to check/correct
     *  @param  Societe     $soc    Object third party
     *  @param  int         $type   0 = customer/prospect , 1 = supplier
	 *  @return int					0 if OK
	 * 								-1 ErrorBadCustomerCodeSyntax
	 * 								-2 ErrorCustomerCodeRequired
	 * 								-3 ErrorCustomerCodeAlreadyUsed
	 * 								-4 ErrorPrefixRequired     
     */
    public function verif($db, &$code, $soc, $type='')
    {
  		global $conf;

		$result=0;
		$code = trim($code);
		if (empty($code) && $this->code_null && empty($conf->global->MAIN_COMPANY_CODE_ALWAYS_REQUIRED))
		{
			$result=0;
		}
		elseif (empty($code) && (! $this->code_null || ! empty($conf->global->MAIN_COMPANY_CODE_ALWAYS_REQUIRED)) )
		{
			$result=-2;
		}
		else
		{
			if ($this->verif_syntax($code) >= 0)
			{
				$is_dispo = $this->verif_dispo($db, $code, $soc, $type);
				//s'il est dispo probleme d'unicité....
				if ($is_dispo = 0)
				{
					$result=-3;
				}
				else
				{
					$result=0;
				}
			}
			else
			{
				if (dol_strlen($code) == 0)
				{
					$result=-2;
				}
				else
				{
					$result=-1;
				}
			}
		}

		dol_syslog(get_class($this)."::verif code=".$code." type=".$type." result=".$result);

		return $result;
    }


/**
	 *		Renvoi si un code est pris ou non (par autre tiers)
	 *
	 *		@param	DoliDB		$db			Handler acces base
	 *		@param	string		$code		Code a verifier
	 *		@param	Societe		$soc		Objet societe
	 *		@param  int		  	$type   	0 = customer/prospect , 1 = supplier
     *  	@return int                 0  CodeAvailable
     *   	                            -1 CodeNotAvailable
     *									-2 ErrorBadTypeCodeSyntax
     *   	                            -3 ErrorSql	 
	 */
	public function verif_dispo($db, $code, $soc, $type = '')
	{
		global $db,$conf;
        
        if ($type == 1) {
                $typecode = 'code_fournisseur';
        }elseif($type == 0) {
                $typecode = 'code_client';
        }else{
            $this->error = 'Bad value for parameter type';
            dol_syslog("mod_codecompta_HAMSTER::verif '".$code."' Other Error Bad value for parameter type : 0 or 1 (value=".$typecode.")");
            return -2;
        }
        dol_syslog("mod_codecompta_HAMSTER::verif code:".$code." type:".$type." typecode:".$typecode." & company=".(! empty($soc->name)?$soc->name:''));


		$sql = "SELECT ".$typecode." FROM ".MAIN_DB_PREFIX."societe";
        $sql.= " WHERE ".$typecode." = '".$code."'";

        $resql=$db->query($sql);
        if ($resql)
        {
            if ($db->num_rows($resql) == 0)
            {
                dol_syslog("mod_codecompta_HAMSTER::verif '".$typecode.'code :'.$code."' available");
                return 0;	// Available
            }
            else
            {
                dol_syslog("mod_codecompta_HAMSTER::verif '".$typecode.'code :'.$code."' not available");
                return -1;	// Not available
            }
        }
        else
        {
            $this->error=$db->error()." sql=".$sql;
            dol_syslog("mod_codecompta_HAMSTER::verif '".$code."'Other Error SQL");
            //return  $type."/".$typecode."/".$sql;		// Error
            return  -3;		// Error
        }

	}



	/**
	 *  Renvoi si un code respecte la syntaxe
	 *
	 *  @param  string      $code       Code a verifier
	 *  @return int                     0 si OK, <0 si KO
	 */
	public function verif_syntax($code)
	{
        // phpcs:enable
		$res = 0;

		if (dol_strlen($code) < $conf->global->COMPANY_HAMSTER_MASK_NBCHARACTER_CUSTOMER)
		{
			$res = -1;
		}
		else
		{
			$res = 0;
		}
		return $res;
	}    

}
